/**
 * This header provides index for the HSDK reset controller.
 */
#ifndef _DT_BINDINGS_RESET_CONTROLLER_SNPS_HSDK
#define _DT_BINDINGS_RESET_CONTROLLER_SNPS_HSDK

#define HSDK_APB_RESET	0
#define HSDK_AXI_RESET	1
#define HSDK_ETH_RESET	2
#define HSDK_USB_RESET	3
#define HSDK_SDIO_RESET	4
#define HSDK_HDMI_RESET	5
#define HSDK_GFX_RESET	6
#define HSDK_DMAC_RESET	7
#define HSDK_EBI_RESET	8

#endif /*_DT_BINDINGS_RESET_CONTROLLER_SNPS_HSDK*/
