/* SPDX-License-Identifier: GPL-2.0 */

#ifndef __DT_BINDINGS_RICHTEK_RT5190A_REGULATOR_H__
#define __DT_BINDINGS_RICHTEK_RT5190A_REGULATOR_H__

/*
 * BUCK/LDO mode constants which may be used in devicetree properties
 * (eg. regulator-allowed-modes).
 * See the manufacturer's datasheet for more information on these modes.
 */

#define RT5190A_OPMODE_AUTO	0
#define RT5190A_OPMODE_FPWM	1

#endif
